#!/usr/bin/env python3
"""
mDNS Service Discoverer
Parameters:
    - IP: Network interface IP to listen on
Additional parameters in comments:
    - timeout: Discovery timeout in seconds
"""
import socket
import struct
import time
import argparse

class MDNSDiscoverer:
    def __init__(self, interface_ip, timeout=5):
        self.interface_ip = interface_ip
        self.timeout = timeout
        self.MCAST_GRP = '224.0.0.251'
        self.MCAST_PORT = 5353
        
    def create_socket(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind(('', self.MCAST_PORT))
        
        # Tell the kernel to join a multicast group
        mreq = struct.pack("4s4s",
                          socket.inet_aton(self.MCAST_GRP),
                          socket.inet_aton(self.interface_ip))
        sock.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
        return sock
        
    def discover(self):
        print(f"Starting mDNS discovery on interface {self.interface_ip}")
        print(f"Listening for {self.timeout} seconds...\n")
        
        sock = self.create_socket()
        sock.settimeout(1)
        
        start_time = time.time()
        services = set()
        
        try:
            while time.time() - start_time < self.timeout:
                try:
                    data, addr = sock.recvfrom(1024)
                    if addr[0] not in services:
                        services.add(addr[0])
                        print(f"Found mDNS service at {addr[0]}:{addr[1]}")
                except socket.timeout:
                    continue
                    
        except KeyboardInterrupt:
            print("\nDiscovery stopped by user")
        finally:
            sock.close()
            
        print(f"\nDiscovery complete. Found {len(services)} unique services.")
        if services:
            print("\nDiscovered Services:")
            for service in sorted(services):
                print(f"- {service}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="mDNS Service Discoverer")
    parser.add_argument("ip", help="Network interface IP to listen on")
    parser.add_argument("--timeout", type=int, default=5,
                      help="Discovery timeout in seconds")
    args = parser.parse_args()
    
    discoverer = MDNSDiscoverer(args.ip, args.timeout)
    discoverer.discover()
